package dev.pfaff.altargui.shader;

import dev.pfaff.glass.opengl.enums.GLVertexAttribPointerType;
import dev.pfaff.glass.opengl.enums.IGLVertexAttribPointerType;
import org.jspecify.annotations.Nullable;

import java.lang.foreign.MemoryLayout;
import java.lang.foreign.ValueLayout;
import java.nio.ByteOrder;
import java.util.Objects;

public record VertexAttribSpec(String name, IGLVertexAttribPointerType type, boolean normalized, int width, @Nullable MemoryLayout customLayout) {
	public static final VertexAttribSpec PADDING_BYTE = new VertexAttribSpec("padding", GLVertexAttribPointerType.BYTE, false, 1);

	public VertexAttribSpec {
		if (normalized && !(type instanceof GLVertexAttribPointerType)) {
			throw new UnsupportedOperationException("Only GLVertexAttribPointerType can be normalized");
		}
	}

	public VertexAttribSpec(String name, IGLVertexAttribPointerType type, boolean normalized, int width) {
		this(name, type, normalized, width, null);
	}

	public VertexAttribSpec(String name, IGLVertexAttribPointerType type, int width) {
		this(name, type, false, width);
	}

	public VertexAttribSpec withNormalized() {
		return new VertexAttribSpec(name, type, true, width, customLayout);
	}

	public VertexAttribSpec withCustomLayout(MemoryLayout layout) {
		return new VertexAttribSpec(name, type, normalized, width, Objects.requireNonNull(layout));
	}

	public VertexAttribSpec withByteOrder(ByteOrder byteOrder) {
		return withCustomLayout(((ValueLayout) Objects.requireNonNull(customLayout)).withOrder(byteOrder));
	}
}
